/*
   Copyright 2009-2010 IEAP CTU
   Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
 */
package cz.ieap.jplugininterface;

/**
 * Interface that contains information about one java plugin such as name, path and
 * objects/interfaces that this plugin exports.
 *
 * @author Daniel Turecek
 * @version 1.0, 4/Mar/2010 12:47:00
 */
public interface JPluginInfoInterface {

    /** object is not assigned to specified device, but it's general for whole plugin*/
    public final static int ALL_DEVICES = -1;

    /**
     * Gets short name of the plugin (e.g. jmpxctrlui) 
     * @return
     */
    public String getName();

    /**
     * Gets  path and file name of the plugin
     * (e.g. /home/user/Pixelman/jplugins/jmpxctrlui.jar)
     * @return plugin path and filename
     */
    public String getPath();

    /**
     * Gets Object (ObjectInfo) with name objectName for all/no device (ALL_DEVICES).
     * @param objectName object name to retrieve
     * @return ObjectInfo or if object not found null.
     */
    public ObjectInfo getObject(String objectName);

    /**
     * Gets object (ObjectInfo) with object name for specified device.
     * @param deviceIndex index of the device or ALL_DEVICES
     * @param objectName name of the object to retrieve
     * @return ObjectInfo or if object not found null.
     */
    public ObjectInfo getObject(int deviceIndex, String objectName);
}
