/*
   Copyright 2009-2010 IEAP CTU
   Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
 */
package cz.ieap.jplugininterface;


/**
 * Plugin Main Interface. Plugin needs to implements this interface in order to
 * be loaded by pixelman. Class implementing this should have only simple contructor.
 *
 * @author Daniel Turecek
 */
public interface JPluginInterface {

    /** version of API at compile time - version this plugin was compiled for */
    public final static int API_VERSION = 222;

    /**
     * <p>Return name of the plugin that implements this interface.
     * This function should return valid value even if it is called
     * before initialize plugin. </p>
     * <p>Plugin name must be lowercase and should contain only these characters: a-z A-Z 0-9 _</p>
     * @return name of the plugin
     */
    public String getPluginName();

    /**
     * Plugin must return it's API version like this: <code> return API_VERSION;</code>
     * @return api version plugin was compiled for
     */
    public int getAPIVersion();

    
    /**
     * Initialize plugin.
     * @param pixelmanFuncsInterface interface containing pixelman functions.
     * @param temporaryDirectory directory where files from jar of plugin will be unpacked.
     *        this means that resources that might be in jar will be in this directory
     * @param nativeLibDirectory directory where native libraries should be, 
     *         if plugin needs them. If null, native libraries are located in temporaryDirectory.
     * @return if plugin is initialized successfully, should return 0.
     *         if error occured should return non-zero value.
     */
    public int initializePlugin(PixelmanFuncsInterface pixelmanFuncsInterface,
                                String temporaryDirectory,
                                String nativeLibDirectory);

    /**
     * Called when JMpxLoader is fully loaded and all plugins are loaded and initialized.
     */
    public void startPlugin();


    /**
     * Called when pixelman exits for plugin to clean up resources etc.
     * @return
     */
    public void finalizePlugin();


    /**
     * Sets JPluginsManagerInterface, which plugin may use to get information about
     * othe plugins. This method is called before initializing of plugin, so it can be
     * used in initializePlugin method.
     * @param pluginsManagerInterface plugin's manager interface
     */
    public void setPluginsManagerInterface(JPluginsManagerInterface pluginsManagerInterface);

}
