/*
   Copyright 2009-2010 IEAP CTU
   Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
 */
package cz.ieap.jplugininterface;

import java.util.List;

/**
 * Interface exporting function of PluginManager for java plugins. 
 * Allowing acces to list of plugins, to their information
 * and to their exported objects/interfaces.
 *
 * @author Daniel Turecek
 * @version 1.0, 4/Mar/2010 11:29:00
 */
public interface JPluginsManagerInterface {

    /**
     * Gets number of loaded plugins
     * @return number of plugins
     */
    public int getPluginInfoCount();

    /**
     * Gets plugin by it's name
     * @param pluginName name of the plugin to get
     * @return JPluginInfo or null if not found
     */
    public JPluginInfoInterface getPluginInfo(String pluginName);

    /**
     * Gets list of all loaded plugins
     * @return list of all pugins (JPluginInfo)
     */
    public List<JPluginInfoInterface> getPluginInfos();

    /**
     * Register new plugin object/interface to share with other plugins
     * @param plugin plugin that is registering this object
     * @param objectInfo  object (ObjectInfo) to be registered
     * @param deviceIndex index of device for which it's registered (or
     * JPluginInfovoid.ALL_DEVICES if for all/none).
     * @return true registered, false error occured
     */
    public boolean registerPluginObject(String plugin, ObjectInfo objectInfo, int deviceIndex);

}
