/*
   Copyright 2009-2010 IEAP CTU
   Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
 */
package cz.ieap.jplugininterface.structs;

/**
 * Class for storing DevInfo structure data
 * @author Daniel Turecek
 */
public class DevInfo {

    public int pixCount;                        // total number of pixels
    public int rowLen;                          // length of row in pixels (e.g 256 for single chip, 512 for quad);
    public int numberOfChips;                   // number of chips
    public int numberOfRows;                    // number of rows in which chips are aligned (e.g. quad has 4 chips, which are in 2 rows)
    public int mpxType;                         // medipix type - MPX_ORIG, MPX_MXR, MPX_TPX
    public String chipboardID;                  // id of chip/chipboard
    public String ifaceName;                    // name of interface
    public int ifaceType;                       // type of the interface (IFT_MUROS, IFT...) (0-62)
    public int ifaceVendor;                     // interface vendor ID (IFV_IEAP, ...) (1-4094)
    public int ifaceSerial;                     // serial number of the interface (1-262142)
    public int chipboardSerial;                 // serial number of the chipboard (1-(64^4-2))

    public long suppAcqModes;                   // supported acq. mode bitwise combinations of ACQMODE_xxx flags
    public boolean suppCallback;                // callback is supported (acq. is finished, triggers)

    public double clockReadout;                 // clock frequency [MHz] for readout
    public double clockTimepix;                 // clock in frequency [MHz] for Timepix information

    // hw timer capabilities
    public double timerMinVal;                  // minimum value of hw timer [s]
    public double timerMaxVal;                  // maximum value of hw timer [s]
    public double timerStep;                    // step of hw timer [s]

    // test pulse capabilities
    public long maxPulseCount;                  // maximum number of pulses that can be sent
    public double maxPulseHeight;               // max pulse height [V]
    public double maxPulsePeriod;               // max period of pulses [s], length of pulse should be period/2

    // ext DAC capabilities
    public double extDacMinV;                   // minimum external DAC voltage
    public double extDacMaxV;                   // maximum external DAC voltage
    public double extDacStep;                   // ext. DAC step size

    public DevInfo(){
        chipboardID = "";
        ifaceName = "";
    }

    @Override
    public String toString(){
        return "[pixCount: "+pixCount+"\n rowLen: "+rowLen+"\n numberOfChips: "+numberOfChips+"\n numberOfRows: "+numberOfRows+
		"\n mpxType: "+mpxType+"\n chipboardID: "+chipboardID+"\n ifaceName: "+ifaceName+"\n ifaceType: "+ifaceType+"\n ifaceVendor: "+ifaceVendor
            +"\n ifaceSerial: "+ifaceSerial+"\n chipboardSerial: "+chipboardSerial+"\n suppacqModes: "+suppAcqModes+
		"\n clockReadout: "+clockReadout+"\n clockTimepix: "+clockTimepix+"\n timerMinVal: "+timerMinVal+"\n timerMaxVal: "+timerMaxVal+
		"\n timerStep: "+timerStep+"\n maxPulseCount: "+maxPulseCount+"\n maxPulseHeight: "+maxPulseHeight+"\n maxPulsePeriod: "+maxPulsePeriod+
		"\n extDacMinV: "+extDacMinV+"\n extDacStep: "+extDacStep+"]";
    }

}
