/*
   Copyright 2009-2010 IEAP CTU
   Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
 */
package cz.ieap.jplugininterface.structs;

import cz.ieap.jplugininterface.callbacks.PluginFuncInterface;

/**
 * Class describing plugin function registered in Manager
 * @author Daniel Turecek
 */
public class ExtFunctionInfo {
    final static int MAX_PARAM = 20;

    public long functionInfoID = 0;       // id of functoin info - filled when called by getRegFuncsList
    public PluginFuncInterface pluginFuncInterface;     // pointer to registered function
    public int paramInCount;             // number of input parameters
    public int paramOutCount;            // number of output parameters
    public ExtParamInfo[] paramsInfoIn;  // input parameters infos
    public ExtParamInfo[] paramsInfoOut; // output parameters infos
    public String pluginName = "";       // plugin name
    public String functionName = "";     // function name
    public String description = "";      // function description
    public long userData;                // custom parameter that will be used for func call, used by Javabridge, any value assigned will be overwritten
    public long flags;                   // combination of FUNCFLAG_XXX
    public long funcID;                   // id of function

    /**
     * Constructor of ExtFunctionInfo.
     * Initialized paramsInfoIn and pamrasInfoOut
     */
    public ExtFunctionInfo(){
        paramsInfoIn = new ExtParamInfo[MAX_PARAM];
        paramsInfoOut = new ExtParamInfo[MAX_PARAM];
    }

    /**
     * Constructor of ExtFunctionInfo.
     * @param aPluginFunInterface  pointer to registered function
     * @param aParamInCount number of input parameters
     * @param aParamOutCount number of output parameters
     * @param aParamsInfoIn input parameters infos
     * @param aParamsInfoOut output parameters infos
     * @param aPluginName plugin name
     * @param aFunctionName function name
     * @param aDescription function desc
     * @param aUserData custom parameter that will be used for func call, used by Javabridge, any value assigned will be overwritten
     * @param aFlags combination of FUNCFLAG_XXX
     * @param aFuncID id of function
     */
    public ExtFunctionInfo(PluginFuncInterface aPluginFunInterface,
                           int aParamInCount,
                           int aParamOutCount,
                           ExtParamInfo[] aParamsInfoIn,
                           ExtParamInfo[] aParamsInfoOut,
                           String aPluginName,
                           String aFunctionName,
                           String aDescription,
                           long aUserData,
                           long aFlags,
                           long aFuncID)
    {
        this();
        pluginFuncInterface = aPluginFunInterface;
        paramInCount = aParamInCount;
        paramOutCount = aParamOutCount;

        // copy params - user could pass smaller or bigger array thatn MAX_PARAM
        if (aParamsInfoIn != null)
            for (int i = 0; i < Math.min(aParamsInfoIn.length, MAX_PARAM); i++)
                paramsInfoIn[i] = aParamsInfoIn[i];

        // copy params - user could pass smaller or bigger array thatn MAX_PARAM
        if (aParamsInfoOut != null)
            for (int i = 0; i < Math.min(aParamsInfoOut.length, MAX_PARAM); i++)
                paramsInfoOut[i] = aParamsInfoOut[i];

        pluginName = aPluginName;
        functionName = aFunctionName;
        description = aDescription;
        userData = aUserData;
        flags = aFlags;
        funcID = aFuncID;
    }

    
}
