/*
   Copyright 2009-2010 IEAP CTU
   Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
 */
package cz.ieap.jplugininterface.structs;

/**
 * Class for storing Mpx3 Acquisition parameters
 * @author Daniel Turecek
 */
public class Mpx3AcqParameters {

    // COUNTER SELECT
    /** counter1 constant for counterSel*/
    public final static int COUNTER1 = 0;
    /** counter2 constant for counterSel*/
    public final static int COUNTER2 = 1;
    /** both counters constant for counterSel*/
    public final static int COUNTER_BOTH = 2;

    // OPERATION MODE
    // single pixel mode + sequential read write
    public final static int SPM_SRW = 0;       
    // single pixel mode + continuous read write
    public final static int SPM_CRW = 1;       
    // charge summing mode + sequential read write
    public final static int CSM_SRW = 2;       
    // charge summing mode + continuous read write
    public final static int CSM_CRW = 3;     
    // spectroscopic + single pixel mode + sequential read write
    public final static int SPECT_SPM_SRW = 4; 
    // spectroscopic + single pixel mode + continuous read write
    public final static int SPECT_SPM_CRW = 5; 
    // spectroscopic + charge summing mode + sequential read write
    public final static int SPECT_CSM_SRW = 6; 
    // spectroscopic + charge summing mode + continuous read write
    public final static int SPECT_CSM_CRW = 7; 

    // COUNTER DEPTH  
    // 1 bit counter depth
    public final static int COUNTER_1B = 0;  
    // 4 bits (Medipix3.0) or 6 bits (Medipix RX) counter depth
    public final static int COUNTER_4B_6B = 1;
    // 12 bits counter depth
    public final static int COUNTER_12B = 2;   
    // 24 bits counter depth
    public final static int COUNTER_24B = 3;   
    
    /** operation mode */
    public int opMode;

    /** depth of counter 1,4,12,24 bits */
    public int counterDepth;

    /** selected counter - first, second, both */
    public int counterSel;

    /** column block selection */
    public int columnBlock;

    /** row block selection */
    public int rowBlock;

    /** if equalize THH bit in OMR is active*/
    public boolean equalizeTHH;

    /** equalization OMR bit*/
    public boolean equalization;

    /** disc Csm Spm OMR bit*/
    public boolean discCsmSpm;

    /** info header OMR bit*/
    public boolean infoHeader;

    /** gain mode 0-3 */
    public int gain;
}
