# Copyright 2009-2012 IEAP CTU
# Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
"""
Module with often used constants and function with medipix devices.
"""
import sys
import time
from jarray import array, zeros
from cz.ieap.pixelman.jmpxloader import Main
from cz.ieap.jplugininterface.structs import HwInfoItem, DevInfo
from cz.ieap.pixelman.shared.mpxutils import HwInfoItemMgr
from cz.ieap.pixelman.shared.buffersutils import Parameters, BOOL, DOUBLE, I16, U16, I32, U32, STRING, BYTE, CHAR, FLOAT, UCHAR
from java.nio import ByteBuffer
from java.nio import ByteOrder;
import pixelman_regfuncs
import jmpxctrlui

pixelman = Main.mMainClass.getPixelmanFuncs()

# data types
TYPE_BOOL   = 0
TYPE_CHAR   = 1
TYPE_UCHAR  = 2
TYPE_BYTE   = 3
TYPE_I16    = 4
TYPE_U16    = 5
TYPE_I32    = 6
TYPE_U32    = 7
TYPE_I32    = 8
TYPE_U32    = 9 
TYPE_FLOAT  = 10 
TYPE_DOUBLE = 11 
TYPE_STRING = 12

# Medipix 2
ORIG_COUNT = 14
ORIG_DELAYN = 0
ORIG_DISC = 1
ORIG_PREAMP = 2
ORIG_SETDISC = 3
ORIG_THS = 4
ORIG_IKRUM = 5
ORIG_ABUFFER = 6
ORIG_VTHH = 7
ORIG_VTHL = 8
ORIG_VFBK = 9
ORIG_VGND = 10
ORIG_BIASLVDSTX = 11
ORIG_REFLVDSTX = 12
ORIG_IKRUMHALF = 13

# MXR
MXR_COUNT = 15
MXR_IKRUM = 0
MXR_DISC = 1
MXR_PREAMP = 2
MXR_BUFFA = 3
MXR_BUFFB = 4
MXR_DELAYN = 5
MXR_THLFINE = 6
MXR_THLCOARSE = 7
MXR_THHFINE = 8
MXR_THHCOARSE = 9
MXR_FBK = 10
MXR_GND = 11
MXR_THS = 12
MXR_BIASLVDS = 13
MXR_REFLVDS = 14

# Timepix
TPX_COUNT = 14
TPX_IKRUM = 0
TPX_DISC = 1
TPX_PREAMP = 2
TPX_BUFFA = 3
TPX_BUFFB = 4
TPX_HIST = 5
TPX_THLFINE = 6
TPX_THLCOARSE = 7
TPX_VCAS = 8
TPX_FBK = 9
TPX_GND = 10
TPX_THS = 11
TPX_BIASLVDS = 12
TPX_REFLVDS = 13

# Medipix3
MPX3_COUNT = 25
MPX3_TH0 = 0
MPX3_TH1 = 1
MPX3_TH2 = 2
MPX3_TH3 = 3
MPX3_TH4 = 4
MPX3_TH5 = 5
MPX3_TH6 = 6
MPX3_TH7 = 7
MPX3_PREAMP = 8
MPX3_IKRUM = 9
MPX3_SHAPER = 10
MPX3_DISC = 11
MPX3_DISC_LS = 12
MPX3_TH_N = 13
MPX3_DAC_PIXEL = 14
MPX3_DELAY = 15
MPX3_TP_BUFF_IN = 16
MPX3_TP_BUFF_OUT = 17
MPX3_RPZ = 18
MPX3_GND = 19
MPX3_TP_REF = 20
MPX3_FBK = 21
MPX3_CAS = 22
MPX3_TP_REFA = 23
MPX3_TP_REFB = 24

JDEVICE_SPECIFIC = "jdevicespecific"


mpxUtilsHwInfoMgrs = {}
jmpxctrluiAcqSerFinishedSet = {}

def jmpxctrluiAcqSerFinished(callbackParam, userData):
    global jmpxctrluiAcqSerFinishedSet
    jmpxctrluiAcqSerFinishedSet[callbackParam] = 2

def getBias(devID):
    """Gets Bias value from device"""
    global mpxUtilsHwInfoMgrs
    if not mpxUtilsHwInfoMgrs.has_key(str(devID)):
        mpxUtilsHwInfoMgrs[str(devID)] = HwInfoItemMgr(pixelman, devID)
    return mpxUtilsHwInfoMgrs[str(devID)].getHwInfoItem("HV")

def setBias(devID, bias):
    """Sets Bias voltage to device"""
    global mpxUtilsHwInfoMgrs
    if not mpxUtilsHwInfoMgrs.has_key(str(devID)):
        mpxUtilsHwInfoMgrs[str(devID)] = HwInfoItemMgr(pixelman, devID)
    rc = mpxUtilsHwInfoMgrs[str(devID)].setHwInfoItem("HV", str(bias))
    return

def getTpxClock(devID):
    """Gets value of Timepix Clock in MHz from devic"""
    global mpxUtilsHwInfoMgrs
    if not mpxUtilsHwInfoMgrs.has_key(str(devID)):
        mpxUtilsHwInfoMgrs[str(devID)] = HwInfoItemMgr(pixelman, devID)
    return mpxUtilsHwInfoMgrs[str(devID)].getHwInfoItem("Timepix clock")

def setTpxClock(devID, clock):
    """Sets Timepix Clock to device"""
    global mpxUtilsHwInfoMgrs
    if not mpxUtilsHwInfoMgrs.has_key(str(devID)):
        mpxUtilsHwInfoMgrs[str(devID)] = HwInfoItemMgr(pixelman, devID)
    return mpxUtilsHwInfoMgrs[str(devID)].setHwInfoItem("Timepix clock", str(clock))

def getFileDevFile(devID):
    """Gets current file name or directory from File Device. In HW specific FileName parameter"""
    global mpxUtilsHwInfoMgrs
    if not mpxUtilsHwInfoMgrs.has_key(str(devID)):
        mpxUtilsHwInfoMgrs[str(devID)] = HwInfoItemMgr(pixelman, devID)
    return mpxUtilsHwInfoMgrs[str(devID)].getHwInfoItem("Data file name")

def setFileDevFile(devID, file):
    """Sets File Device file name or directory. In HW specific it is the FileName parameter"""
    global mpxUtilsHwInfoMgrs
    if not mpxUtilsHwInfoMgrs.has_key(str(devID)):
        mpxUtilsHwInfoMgrs[str(devID)] = HwInfoItemMgr(pixelman, devID)
    return mpxUtilsHwInfoMgrs[str(devID)].setHwInfoItem("Data file name", file)

def getChipID(devID):
    """Gets chipboard ID of the conencted chip. String such as: G02-W0082"""
    devInfo = DevInfo()
    pixelman.mpxCtrlGetDevInfo(devID, devInfo)
    return devInfo.chipboardID

def getChipType(devID):
    """Gets type of the connected chip. 1=MPX_ORIG, 2=MPX_MXR, 3=MPX_TPX, 4=MPX_3"""
    devInfo = DevInfo()
    pixelman.mpxCtrlGetDevInfo(devID, devInfo)
    return devInfo.mpxType

def waitForAcqFinish(devID):
    """Waits fro the acquisition started from jmpxctrlui.StartAcq() function is finished.
    Used to block script execution until acquisition is finished."""
    global jmpxctrluiAcqSerFinishedSet
    if len(jmpxctrluiAcqSerFinishedSet) == 0:
        pixelman.registerCallback("jmpxctrlui", "AcqSerFinished", jmpxctrluiAcqSerFinished, 0);

    jmpxctrluiAcqSerFinishedSet[devID] = 1
    while jmpxctrluiAcqSerFinishedSet[devID] == 1: 
        time.sleep(0.2)

def startAcq(devID):
    """Calls jmpxctrlui.StartAcq() fucntion and starts acqusition in specified device"""
    jmpxctrlui.StartAcq(devID)

def setDAC(devID, chipType, dacNum, value):
    """Set DAC value to the device
    chipType - type of the connected chip (1=MPX_ORIG, 2=MPX_MXR, 3=MPX_TPX, 4=MPX_3")
    dacNum - index of the DAC to be changed (constants such as mpxutils.MXR_THLFINE, mpxutils.MPX3_TH1, ...) 
    value - value of the DAC (same as in Dac Panel)
    """
    dacCount = TPX_COUNT 
    if chipType == pixelman.MPX_MXR:
        dacCount = MXR_COUNT 
    if chipType == pixelman.MPX_3:
        dacCount = MPX3_COUNT

    dacs = zeros(dacCount, 'h')
    rc = pixelman.mpxCtrlGetDACs(devID, dacs, dacCount, 0)
    if rc != 0: 
        sys.exit("Could not get DACs: "+str(rc))

    dacs[dacNum] = value
    rc = pixelman.mpxCtrlSetDACs(devID, dacs, dacCount, 0)
    if rc != 0: 
        sys.exit("Could not set DACs: "+str(rc))
    return

def setThreshold(devID, chipType, value):
    """Sets detector threshold DAC. Automaticaly chooses corret DAC index according to detector type
    chipType - type of the connected chip (1=MPX_ORIG, 2=MPX_MXR, 3=MPX_TPX, 4=MPX_3")
    value - value of the threshold (same as in Dac Panel)
    """
    dacCount = TPX_COUNT 
    thlIdx = TPX_THLFINE

    if chipType == pixelman.MPX_MXR:
        dacCount = MXR_COUNT 
        thlIdx = MXR_THLFINE
    if chipType == pixelman.MPX_3:
        dacCount = MPX3_COUNT
        thlIdx = MPX3_TH0
    if chipType == pixelman.MPX_ORIG:
        dacCount = ORIG_COUNT
        thlIdx = ORIG_VTHL

    dacs = zeros(dacCount, 'h')
    rc = pixelman.mpxCtrlGetDACs(devID, dacs, dacCount, 0)
    if rc != 0: 
        sys.exit("Could not get DACs: "+str(rc))

    dacs[thlIdx] = value
    rc = pixelman.mpxCtrlSetDACs(devID, dacs, dacCount, 0)
    if rc != 0: 
        sys.exit("Could not set DACs: "+str(rc))
    return

def getLastAcqFrameID(devID):
    """Gets Frame ID of last frame in last finished acquisition series measured by the device"""
    frameFilled = zeros(1, 'l')
    frameID = zeros(1, 'i')

    rc = pixelman.mpxCtrlGetAcqInfo(devID, None, None, None, frameFilled);
    if rc != 0:
        sys.exit("Could not get acq info: "+str(rc))

    rc = pixelman.mpxCtrlGetFrameID(devID, frameFilled[0]-1, frameID);
    if rc != 0:
        sys.exit("Could not get frame id: "+str(rc))

    return frameID[0]

