# Copyright 2009-2012 IEAP CTU
# Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
"""
This module crates dynamically python functions for all exported function of pixelman plugins (that are normally called through pixelman.callItemFunc)
"""
from cz.ieap.pixelman.jmpxloader import Main
from cz.ieap.jplugininterface.structs import ExtFunctionInfo, ExtParamInfo, DataTypes
from cz.ieap.pixelman.shared.buffersutils import Parameters, BOOL, DOUBLE, I16, U16, I32, U32, I64, U64, STRING, BYTE, CHAR, FLOAT, UCHAR
from jarray import array, zeros

pixelman = Main.mMainClass.getPixelmanFuncs()

types = ["BOOL", "CHAR", "UCHAR", "BYTE", "I16", "U16", "I32", "U32", "I64", "U64", "FLOAT", "DOUBLE", "STRING", "LAST"]
jythonTypes = {"BOOL":"z", "BYTE":"b", "I16":"h", "U16":"i", "I32":"i", "U32":"l", "I64" : "l", "U64" : "l", "FLOAT":"f", "DOUBLE":"d"} 

def importCode(code,name,add_to_sys_modules=0):
    import sys,imp

    module = imp.new_module(name)

    exec code in module.__dict__
    if add_to_sys_modules:
        sys.modules[name] = module

    return module


def initRegFuncs():
    funcs = pixelman.getRegFuncsList()
    code = ""
    plugin = ""
    moduleName = ""
    codePrefix = \
    """
from cz.ieap.pixelman.jmpxloader import Main
from cz.ieap.jplugininterface.structs import ExtFunctionInfo, ExtParamInfo
from cz.ieap.pixelman.shared.buffersutils import Parameters, BOOL, DOUBLE, I16, U16, I32, U32, I64, U64, STRING, BYTE, CHAR, FLOAT, UCHAR
from jarray import array, zeros
pixelman = Main.mMainClass.getPixelmanFuncs()
"""
    for funcItem in funcs:
        moduleName = funcItem.pluginName
        funcName = funcItem.functionName
        origFunc = funcName
        funcName = funcName.replace(" ", "_")
        funcName = funcName.replace(".", "_")
        funcName = funcName.replace("-", "_")

        if plugin == "": 
            plugin = moduleName 

        if plugin != moduleName:
            importCode(codePrefix+code, plugin, 1)
            code = ""
            plugin = moduleName

        funcParams = ""
        inParams = ""
        retParams = ""
        outParams = ""
        for i in range(funcItem.paramInCount):
            count = funcItem.paramsInfoIn[i].count
            parName = "inParam%d" % i
            funcParams += parName+", " 
            inParams += types[funcItem.paramsInfoIn[i].type]+"("+parName+"), "

        paramCode = ""
        for i in range(funcItem.paramOutCount):
            count = funcItem.paramsInfoOut[i].count
            parName = "outParam%d" % i
            retParams += ", "+ parName
            parType = types[funcItem.paramsInfoOut[i].type]
            if parType in jythonTypes and count > 1:
                outParams += types[funcItem.paramsInfoOut[i].type]+"(zeros("+str(count)+", '"+jythonTypes[parType]+"')), "
            else:
                if count == 1:
                    outParams += types[funcItem.paramsInfoOut[i].type]+"(), "
                else:
                    outParams += types[funcItem.paramsInfoOut[i].type]+"("+str(count)+"), "
            if count == 1 or parType == "CHAR" or parType == "UCHAR" or parType == "STRING":
                paramCode += "    %s = outParams.getParams()[%d].getValue()\n" % (parName, i)
            else:
                paramCode += "    %s = outParams.getParams()[%d].getValues()\n" % (parName, i)

        outParams = outParams[:-2]
        retParams = "rc"+retParams

        if funcItem.paramOutCount > 0: 
            outParam = "outParams.getBuffer()"
        else: 
            outParam = "None"
            
        code += "def %s(%s):\n" % (funcName, funcParams[:-2])
        if outParams != "":
            code +="    outParams = Parameters([%s])\n" % outParams 
        code +="    rc = pixelman.callFuncItem(\"%s\", \"%s\", Parameters([%s]).getBuffer(), %s)\n" % (moduleName, origFunc, inParams[:-2], outParam)
        #code +="    if rc != 0: return -1\n"
        code += paramCode
        code +="    return "+retParams+"\n\n"

    if plugin != "":
        importCode(codePrefix+code, plugin, 1)


if __name__ != '__main__':
    initRegFuncs()

        
