# Copyright 2009-2012 IEAP CTU
# Author: Daniel Turecek (daniel.turecek@utef.cvut.cz)
"""
This module is wrapper around Plot2D class to simplify visualisation of plots.
"""
from cz.ieap.jplugininterface.structs import DataTypes
from cz.ieap.pixelman.shared.plot2D import Plot2D, PlotLine
from javax.swing import JFrame, JPanel, JScrollPane
from net.miginfocom.swing import MigLayout
from jarray import array, zeros

class Plot:
    """Wrapper class around Plot2D java class. It automatically creates window with Plot object"""

    def __init__(self,plotName="Plot", width=500, height=300):
        self.frame = JFrame(title=plotName, size=(width,height))
        self.frame.setLocationRelativeTo(None)
        self.plot = Plot2D(1)
        self.plot.addEmptyLine(0)
        self.frame.add(JScrollPane(self.plot))
        self.frame.setVisible(True)

    def setAxisNames(self, xAxis, yAxis):
        """Set names of axis x and y"""
        self.plot.setAxisNames(xAxis, yAxis)

    def setTitle(self, title):
        """Sets title of the plot"""
        self.plot.setGraphTitle(title)
    
    def addPoint(self,x,y, line=0, layer=0):
        """Adds point to line"""
        self.plot.addPoint(x,y,line,layer,True)

    def addEmptyLine(self, layer=0):
        """Adds empty line so that it can be used by addPoint fuction"""
        self.plot.addEmptyLine(layer)

    def setRangeX(self, fromx, tox):
        """Sets X axis range"""
        self.plot.setRangeX(fromx, tox)

    def setRangeY(self, fromy, toy):
        """Sets Y axis range"""
        self.plot.setRangeY(fromy, toy)

    def setRange(self, fromx, tox, fromy, toy):
        """Sets Range for both axis"""
        self.plot.setRange(fromx, tox, fromy, toy)

    def setAutoRange(self, xaxis, yaxis):
        """Sets automatical range for axis"""
        self.plot.setAutoRangeX(xaxis)
        self.plot.setAutoRangeY(yaxis)

    def setAutoRangeX(self, auto):
        """Sets automatical range for X axis"""
        self.plot.setAutoRangeX(auto)

    def setAutoRangeY(self, auto):
        """Sets automatical range for Y axis"""
        self.plot.setAutoRangeY(auto)

    def clear(self):
        """Clear all lines in the plot"""
        self.plot.clearAllLayers()
        
    def getPlot(self):
        """Get directo access to Plot2D object"""
        return self.plot

    def setWindowTitle(self, title):
        """Sets title of the plot window"""
        self.frame.setTitle(title)

        
